import { CodeCell } from '@jupyterlab/cells';
import { colabCommand, markdownBuilder } from './colabCommand';
import {
  CommandType,
  COLAB_CLASS_NAME,
  TOOLBAR_BUTTON_CONTAINER_CLASS_NAME,
  CELL_SHOW_CODE_CLASS_NAME,
  DISPLAY_MODE_BOTH,
  DISPLAY_MODE_CODE,
  DISPLAY_MODE_FORM,
  DISPLAY_MODE_PREFIX,
  CELL_CLASS_NAME,
  TITLE_CLASS_NAME,
  DisplayMode,
  TOOLBAR_INJECTION_SELECTOR
} from './constants';

export class ColabCodeCell extends CodeCell {
  constructor(options: CodeCell.IOptions) {
    super(options);
    this.processColabElement(true);
    this.setDisplayMode(this.displayMode);
    this.model.contentChanged.connect(() => {
      if (!this.colabEditing) {
        this.processColabElement();
      }
    });
  }

  private colabEditing = false;
  private displayMode: DisplayMode = DisplayMode.Both;

  private setColabEditing(value: boolean) {
    this.colabEditing = value;
  }

  private setDisplayMode(mode: DisplayMode) {
    this.displayMode = mode;
    this.node.classList.remove(DISPLAY_MODE_CODE);
    this.node.classList.remove(DISPLAY_MODE_FORM);
    this.node.classList.remove(DISPLAY_MODE_BOTH);
    this.node.classList.add(DISPLAY_MODE_PREFIX + this.displayMode);
    this.editorWidget.update();
  }

  public clearInjectedToolbarBtns(): void {
    this.node
      .querySelector('.' + TOOLBAR_BUTTON_CONTAINER_CLASS_NAME)
      ?.remove();
  }

  public injectToolbarBtns(): void {
    if (!this.node.querySelector(TOOLBAR_INJECTION_SELECTOR)) {
      setTimeout(() => this.injectToolbarBtns(), 100);
    } else {
      if (!this.node.querySelector('.' + TOOLBAR_BUTTON_CONTAINER_CLASS_NAME)) {
        this.node.querySelector(TOOLBAR_INJECTION_SELECTOR)?.prepend(
          colabCommand.toolbarBtns(
            TOOLBAR_BUTTON_CONTAINER_CLASS_NAME,
            () => {
              if (this.displayMode === DisplayMode.Form) {
                this.setDisplayMode(DisplayMode.Both);
              } else {
                this.setDisplayMode(DisplayMode.Form);
              }
            },
            () => {
              if (this.displayMode === DisplayMode.Code) {
                this.setDisplayMode(DisplayMode.Both);
              } else {
                this.setDisplayMode(DisplayMode.Code);
              }
            }
          )
        );
      }
    }
  }

  private processColabElement(init = false) {
    const colabContainerElem = document.createElement('div');
    colabContainerElem.classList.add(COLAB_CLASS_NAME);

    const code = this.model.value.text;
    const colabElems = [];
    code
      .trim()
      .split('\n')
      .forEach((line, i) => {
        const commandType = colabCommand.match(line);

        // remove the injected title if the first non empty line is not a title
        if (i === 0 && (!commandType || commandType !== CommandType.Title)) {
          this.node.querySelector('.' + TITLE_CLASS_NAME)?.remove();
        }

        if (commandType) {
          // only support title as the first non empty title
          if (i !== 0 && commandType === CommandType.Title) {
            return;
          }

          if (i === 0 && commandType === CommandType.Title) {
            // display mode can only be set on first line title
            // also it will only be considered during init, to keep the same behavior as colab
            // after init, changing display mode won't do anything
            const displayMode = colabCommand.parseDisplayMode(line);
            if (displayMode) {
              init && this.setDisplayMode(displayMode.displayMode);
              line = displayMode.newLine;
            }
          }

          if (commandType === CommandType.Markdown) {
            markdownBuilder.add(colabCommand.trim(commandType, line) as string);
          } else {
            if (markdownBuilder.get()) {
              colabElems.push(markdownBuilder.build());
              markdownBuilder.clear();
            }

            const builtElem = colabCommand.build(
              commandType,
              line,
              i,
              (newLine: string) => {
                const newText = this.model.value.text.split('\n');
                newText[i] = newLine;
                this.model.value.text = newText.join('\n');
                this.update();
              },
              (value: boolean) => this.setColabEditing(value)
            );

            if (commandType === CommandType.Title) {
              this.node.querySelector('.' + TITLE_CLASS_NAME)?.remove();
              builtElem && this.node.prepend(builtElem);
            } else {
              colabElems.push(builtElem);
            }
          }
        } else {
          // if this is a non-markdown line, check and build the markdown we have
          if (markdownBuilder.get()) {
            colabElems.push(markdownBuilder.build());
            markdownBuilder.clear();
          }
        }
      });

    // if the last line(s) are markdown, we have to trigger the build here
    if (markdownBuilder.get()) {
      colabElems.push(markdownBuilder.build());
      markdownBuilder.clear();
    }

    // filter and build the final form container elemenr
    colabElems
      .filter(elem => !!elem)
      .forEach(elem => colabContainerElem.appendChild(elem));
    this.node.querySelector(`.${COLAB_CLASS_NAME}`)?.remove();
    if (colabContainerElem.childNodes.length > 0) {
      this.injectToolbarBtns();
      this.setDisplayMode(this.displayMode);
      colabContainerElem.appendChild(
        colabCommand.showCodeBtn(CELL_SHOW_CODE_CLASS_NAME, () =>
          this.setDisplayMode(DisplayMode.Both)
        )
      );
      this.node
        .querySelector('.jp-Cell-inputWrapper')
        ?.appendChild(colabContainerElem);
      this.node.classList.add(CELL_CLASS_NAME);
    } else {
      this.node.classList.remove(CELL_CLASS_NAME);
      this.node.classList.remove(DISPLAY_MODE_CODE);
      this.node.classList.remove(DISPLAY_MODE_FORM);
      this.node.classList.remove(DISPLAY_MODE_BOTH);
      this.clearInjectedToolbarBtns();
    }
  }
}
