import { colabCommandBuilder } from './builder';
import { BuildTargetType, ParamType } from '../constants';

const update = jest.fn((value: string) => value);
const setEditing = jest.fn((isEditing: boolean) => isEditing);

describe('test builder', () => {
  beforeEach(() => {
    jest.resetAllMocks();
  });
  test('should build the value list', () => {
    const elem = colabCommandBuilder(
      BuildTargetType.ValueList,
      {
        config: {
          type: ParamType.String
        },
        name: 'aa',
        valueList: ['1', '2', '3']
      },
      update,
      setEditing
    );
    expect(elem).toBeTruthy();
    expect(elem?.querySelectorAll('option').length).toBe(3);
    elem?.querySelectorAll('option')[1].setAttribute('selected', 'selected');
    elem?.querySelector('select')?.dispatchEvent(new Event('change'));
    expect(update.mock.calls.length).toBe(1);
    expect(update.mock.calls[0][0]).toBe('2');
    expect(setEditing.mock.calls.length).toBe(2);
    expect(setEditing.mock.calls[0][0]).toBe(true);
    expect(setEditing.mock.calls[1][0]).toBe(false);
  });

  test('should build the value list with raw', () => {
    const elem = colabCommandBuilder(
      BuildTargetType.ValueList,
      {
        config: {
          type: ParamType.Raw
        },
        name: 'aa',
        valueList: ['1', '2', '3']
      },
      update,
      setEditing
    );
    expect(elem).toBeTruthy();
    expect(elem?.querySelectorAll('option').length).toBe(3);
    elem?.querySelectorAll('option')[1].setAttribute('selected', 'selected');
    elem?.querySelector('select')?.dispatchEvent(new Event('change'));
    expect(update.mock.calls.length).toBe(1);
    expect(update.mock.calls[0][0]).toBe('2');
    expect(setEditing.mock.calls.length).toBe(2);
    expect(setEditing.mock.calls[0][0]).toBe(true);
    expect(setEditing.mock.calls[1][0]).toBe(false);
  });

  test('should build the value list with allow-input', () => {
    const elem = colabCommandBuilder(
      BuildTargetType.ValueList,
      {
        config: {
          type: ParamType.String,
          allowInput: true
        },
        name: 'aa',
        valueList: ['1', '2', '3']
      },
      update,
      setEditing
    );
    expect(elem).toBeTruthy();
    expect(elem?.querySelectorAll('option').length).toBe(3);
    expect(elem?.querySelectorAll('input').length).toBe(1);
    elem?.querySelectorAll('option')[1].setAttribute('selected', 'selected');
    elem?.querySelector('select')?.dispatchEvent(new Event('change'));
    expect(update.mock.calls.length).toBe(1);
    expect(update.mock.calls[0][0]).toBe('2');
    expect(setEditing.mock.calls.length).toBe(2);
    expect(setEditing.mock.calls[0][0]).toBe(true);
    expect(setEditing.mock.calls[1][0]).toBe(false);

    const inputElem = elem?.querySelector('input');
    expect(inputElem).toBeTruthy();
    inputElem && inputElem.dispatchEvent(new Event('focus'));
    inputElem && (inputElem.value = '3');
    inputElem && inputElem.dispatchEvent(new Event('input'));
    inputElem && inputElem.dispatchEvent(new Event('blur'));
    expect(update.mock.calls.length).toBe(2);
    expect(update.mock.calls[1][0]).toBe('3');
    expect(setEditing.mock.calls.length).toBe(4);
  });

  test('should build the value list with allow-input and raw', () => {
    const elem = colabCommandBuilder(
      BuildTargetType.ValueList,
      {
        config: {
          type: ParamType.Raw,
          allowInput: true
        },
        name: 'aa',
        valueList: ['1', '2', '3']
      },
      update,
      setEditing
    );
    expect(elem).toBeTruthy();
    expect(elem?.querySelectorAll('option').length).toBe(3);
    expect(elem?.querySelectorAll('input').length).toBe(1);
    elem?.querySelectorAll('option')[1].setAttribute('selected', 'selected');
    elem?.querySelector('select')?.dispatchEvent(new Event('change'));
    expect(update.mock.calls.length).toBe(1);
    expect(update.mock.calls[0][0]).toBe('2');
    expect(setEditing.mock.calls.length).toBe(2);
    expect(setEditing.mock.calls[0][0]).toBe(true);
    expect(setEditing.mock.calls[1][0]).toBe(false);

    const inputElem = elem?.querySelector('input');
    expect(inputElem).toBeTruthy();
    inputElem && inputElem.dispatchEvent(new Event('focus'));
    inputElem && (inputElem.value = '3');
    inputElem && inputElem.dispatchEvent(new Event('input'));
    inputElem && inputElem.dispatchEvent(new Event('blur'));
    expect(update.mock.calls.length).toBe(2);
    expect(update.mock.calls[1][0]).toBe('3');
    expect(setEditing.mock.calls.length).toBe(4);
  });

  test('should build the input with type = string', () => {
    const elem = colabCommandBuilder(
      BuildTargetType.InputStringOrRaw,
      {
        config: {
          type: ParamType.String
        },
        name: 'aa'
      },
      update,
      setEditing
    ) as HTMLInputElement;
    expect(elem).toBeTruthy();
    elem && elem.dispatchEvent(new Event('focus'));
    elem && (elem.value = '3');
    elem && elem.dispatchEvent(new Event('input'));
    expect(update.mock.calls.length).toBe(1);
    expect(update.mock.calls[0][0]).toBe('3');
    expect(setEditing.mock.calls.length).toBe(1);
    expect(setEditing.mock.calls[0][0]).toBe(true);
    elem && elem.dispatchEvent(new Event('blur'));
    expect(setEditing.mock.calls.length).toBe(2);
    expect(setEditing.mock.calls[1][0]).toBe(false);
  });

  test('should build the input with type = raw', () => {
    const elem = colabCommandBuilder(
      BuildTargetType.InputStringOrRaw,
      {
        config: {
          type: ParamType.Raw
        },
        name: 'aa',
        value: '123',
        rawValue: '"123"'
      },
      update,
      setEditing
    ) as HTMLInputElement;
    expect(elem).toBeTruthy();
    expect(elem?.value).toBe('"123"');
    elem && elem.dispatchEvent(new Event('focus'));
    elem && (elem.value = '3');
    elem && elem.dispatchEvent(new Event('input'));
    expect(update.mock.calls.length).toBe(1);
    expect(update.mock.calls[0][0]).toBe('3');
    expect(setEditing.mock.calls.length).toBe(1);
    expect(setEditing.mock.calls[0][0]).toBe(true);
    elem && elem.dispatchEvent(new Event('blur'));
    expect(setEditing.mock.calls.length).toBe(2);
    expect(setEditing.mock.calls[1][0]).toBe(false);
  });

  test('should build the input with type = raw and no value', () => {
    const elem = colabCommandBuilder(
      BuildTargetType.InputStringOrRaw,
      {
        config: {
          type: ParamType.Raw
        },
        name: 'aa'
      },
      update,
      setEditing
    ) as HTMLInputElement;
    expect(elem).toBeTruthy();
    expect(elem?.value).toBe('');
    elem && elem.dispatchEvent(new Event('focus'));
    elem && (elem.value = '3');
    elem && elem.dispatchEvent(new Event('input'));
    expect(update.mock.calls.length).toBe(1);
    expect(update.mock.calls[0][0]).toBe('3');
    expect(setEditing.mock.calls.length).toBe(1);
    expect(setEditing.mock.calls[0][0]).toBe(true);
    elem && elem.dispatchEvent(new Event('blur'));
    expect(setEditing.mock.calls.length).toBe(2);
    expect(setEditing.mock.calls[1][0]).toBe(false);
  });

  test('should build the input with type = number', () => {
    const elem = colabCommandBuilder(
      BuildTargetType.InputNumberOrIntegerOrDate,
      {
        config: {
          type: ParamType.Number
        },
        name: 'aa',
        rawValue: '123.4'
      },
      update,
      setEditing
    ) as HTMLInputElement;
    expect(elem).toBeTruthy();
    expect(elem?.value).toBe('123.4');
    elem && elem.dispatchEvent(new Event('focus'));
    elem && (elem.value = '3');
    elem && elem.dispatchEvent(new Event('input'));
    expect(update.mock.calls.length).toBe(1);
    expect(update.mock.calls[0][0]).toBe('3');
    expect(setEditing.mock.calls.length).toBe(1);
    expect(setEditing.mock.calls[0][0]).toBe(true);
    elem && elem.dispatchEvent(new Event('blur'));
    expect(setEditing.mock.calls.length).toBe(2);
    expect(setEditing.mock.calls[1][0]).toBe(false);
  });

  test('should build the input with type = integer', () => {
    const elem = colabCommandBuilder(
      BuildTargetType.InputNumberOrIntegerOrDate,
      {
        config: {
          type: ParamType.Integer
        },
        name: 'aa',
        rawValue: '123'
      },
      update,
      setEditing
    ) as HTMLInputElement;
    expect(elem).toBeTruthy();
    expect(elem?.value).toBe('123');
    elem && elem.dispatchEvent(new Event('focus'));
    elem && (elem.value = '3');
    elem && elem.dispatchEvent(new Event('input'));
    expect(update.mock.calls.length).toBe(1);
    expect(update.mock.calls[0][0]).toBe('3');
    expect(setEditing.mock.calls.length).toBe(1);
    expect(setEditing.mock.calls[0][0]).toBe(true);
    elem && elem.dispatchEvent(new Event('blur'));
    expect(setEditing.mock.calls.length).toBe(2);
    expect(setEditing.mock.calls[1][0]).toBe(false);
  });

  test('should build the input with type = date', () => {
    const elem = colabCommandBuilder(
      BuildTargetType.InputNumberOrIntegerOrDate,
      {
        config: {
          type: ParamType.Date
        },
        name: 'aa',
        value: '2003-10-03'
      },
      update,
      setEditing
    ) as HTMLInputElement;
    expect(elem).toBeTruthy();
    expect(elem?.value).toBe('2003-10-03');
    elem && elem.dispatchEvent(new Event('focus'));
    elem && (elem.value = '2003-10-05');
    elem && elem.dispatchEvent(new Event('input'));
    expect(update.mock.calls.length).toBe(1);
    expect(update.mock.calls[0][0]).toBe('2003-10-05');
    expect(setEditing.mock.calls.length).toBe(1);
    expect(setEditing.mock.calls[0][0]).toBe(true);
    elem && elem.dispatchEvent(new Event('blur'));
    expect(setEditing.mock.calls.length).toBe(2);
    expect(setEditing.mock.calls[1][0]).toBe(false);
  });

  test('should build the checkbox', () => {
    const elem = colabCommandBuilder(
      BuildTargetType.Checkbox,
      {
        config: {
          type: ParamType.Date
        },
        name: 'aa',
        rawValue: 'True'
      },
      update,
      setEditing
    ) as HTMLInputElement;
    expect(elem).toBeTruthy();
    expect(elem?.checked).toBe(true);
    elem && (elem.checked = false);
    elem?.dispatchEvent(new Event('change'));
    expect(update.mock.calls.length).toBe(1);
    expect(update.mock.calls[0][0]).toBe('False');
    expect(setEditing.mock.calls.length).toBe(2);
    expect(setEditing.mock.calls[0][0]).toBe(true);
    expect(setEditing.mock.calls[1][0]).toBe(false);

    elem && (elem.checked = true);
    elem?.dispatchEvent(new Event('change'));
    expect(update.mock.calls.length).toBe(2);
    expect(update.mock.calls[1][0]).toBe('True');
    expect(setEditing.mock.calls.length).toBe(4);
  });

  test('should build the slider', () => {
    const elem = colabCommandBuilder(
      BuildTargetType.Slider,
      {
        config: {
          type: ParamType.Slider
        },
        name: 'aa',
        rawValue: '20'
      },
      update,
      setEditing
    ) as HTMLInputElement;
    expect(elem).toBeTruthy();
    expect(elem?.value).toBe('20');
    elem && elem.dispatchEvent(new Event('focus'));
    elem && (elem.value = '30');
    elem && elem.dispatchEvent(new Event('input'));
    expect(update.mock.calls.length).toBe(1);
    expect(update.mock.calls[0][0]).toBe('30');
    expect(setEditing.mock.calls.length).toBe(1);
    expect(setEditing.mock.calls[0][0]).toBe(true);
    elem && elem.dispatchEvent(new Event('blur'));
    expect(setEditing.mock.calls.length).toBe(2);
    expect(setEditing.mock.calls[1][0]).toBe(false);
  });

  test('should build the slider with no value', () => {
    const elem = colabCommandBuilder(
      BuildTargetType.Slider,
      {
        config: {
          type: ParamType.Slider,
          min: 10,
          max: 20,
          step: 1
        },
        name: 'aa'
      },
      update,
      setEditing
    ) as HTMLInputElement;
    expect(elem).toBeTruthy();
    expect(elem?.value).toBe('10');
    elem && elem.dispatchEvent(new Event('focus'));
    elem && (elem.value = '15');
    elem && elem.dispatchEvent(new Event('input'));
    expect(update.mock.calls.length).toBe(1);
    expect(update.mock.calls[0][0]).toBe('15');
    expect(setEditing.mock.calls.length).toBe(1);
    expect(setEditing.mock.calls[0][0]).toBe(true);
    elem && elem.dispatchEvent(new Event('blur'));
    expect(setEditing.mock.calls.length).toBe(2);
    expect(setEditing.mock.calls[1][0]).toBe(false);
  });

  test('should build the slider with no value and no min', () => {
    const elem = colabCommandBuilder(
      BuildTargetType.Slider,
      {
        config: {
          type: ParamType.Slider,
          max: 20,
          step: 1
        },
        name: 'aa'
      },
      update,
      setEditing
    ) as HTMLInputElement;
    expect(elem).toBeTruthy();
    expect(elem?.value).toBe('20');
    elem && elem.dispatchEvent(new Event('focus'));
    elem && (elem.value = '15');
    elem && elem.dispatchEvent(new Event('input'));
    expect(update.mock.calls.length).toBe(1);
    expect(update.mock.calls[0][0]).toBe('15');
    expect(setEditing.mock.calls.length).toBe(1);
    expect(setEditing.mock.calls[0][0]).toBe(true);
    elem && elem.dispatchEvent(new Event('blur'));
    expect(setEditing.mock.calls.length).toBe(2);
    expect(setEditing.mock.calls[1][0]).toBe(false);
  });

  test('should build the slider with no value and no min and no max', () => {
    const elem = colabCommandBuilder(
      BuildTargetType.Slider,
      {
        config: {
          type: ParamType.Slider,
          step: 1
        },
        name: 'aa'
      },
      update,
      setEditing
    ) as HTMLInputElement;
    expect(elem).toBeTruthy();
    expect(elem?.value).toBe('0');
    elem && elem.dispatchEvent(new Event('focus'));
    elem && (elem.value = '15');
    elem && elem.dispatchEvent(new Event('input'));
    expect(update.mock.calls.length).toBe(1);
    expect(update.mock.calls[0][0]).toBe('15');
    expect(setEditing.mock.calls.length).toBe(1);
    expect(setEditing.mock.calls[0][0]).toBe(true);
    elem && elem.dispatchEvent(new Event('blur'));
    expect(setEditing.mock.calls.length).toBe(2);
    expect(setEditing.mock.calls[1][0]).toBe(false);
  });
});
