import {
  ParamObj,
  ParamType,
  BuildTargetType,
  PARAM_EXTEND_ELEMENT_CLASS_NAME,
  PARAM_ALLOW_INPUT_INPUT,
  SELECT_WRAPPER_CLASS_NAME
} from '../constants';
import pubsub from '../utils/pubsub';

export const colabCommandBuilder = (
  targetType: BuildTargetType,
  paramObj: ParamObj,
  update: (value: string) => void,
  setColabEditing: (isEditing: boolean) => void,
  key?: number
): HTMLElement | undefined => {
  if (targetType === BuildTargetType.ValueList) {
    const selectElemWrapper = document.createElement('div');
    selectElemWrapper.classList.add(PARAM_EXTEND_ELEMENT_CLASS_NAME);
    selectElemWrapper.classList.add(SELECT_WRAPPER_CLASS_NAME);
    const selectElem = document.createElement('select');
    selectElem.classList.add(PARAM_EXTEND_ELEMENT_CLASS_NAME);
    paramObj.valueList &&
      paramObj.valueList.forEach((value, i) => {
        const optionElem = document.createElement('option');
        optionElem.innerHTML = value as string;
        selectElem.appendChild(optionElem);
      });
    selectElem.value =
      paramObj.config.type === ParamType.String
        ? paramObj.value ?? ''
        : paramObj.rawValue ?? '';

    selectElem.addEventListener('change', e => {
      setColabEditing(true); // avoid the update() to trigger the unwanted colab cell rendering

      let value;
      if (paramObj.config.type === ParamType.Raw && paramObj.valueList) {
        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
        value =
          paramObj.valueList?.[(e.target as HTMLSelectElement).selectedIndex];
      } else {
        value = (e.target as HTMLSelectElement).value;
      }
      update(value);
      // make sure to sync the value with the input
      pubsub.publish(`select-value${(key ?? '').toString()}`, value);
      setColabEditing(false);
    });

    selectElemWrapper.appendChild(selectElem);

    // if allow-input is true, add the input and arrow button on top of the select elem
    if (paramObj.config.allowInput) {
      const inputElem = document.createElement('input');
      inputElem.classList.add(PARAM_EXTEND_ELEMENT_CLASS_NAME);
      inputElem.classList.add(PARAM_ALLOW_INPUT_INPUT);
      inputElem.value =
        paramObj.config.type === ParamType.String
          ? paramObj.value ?? ''
          : paramObj.rawValue ?? '';

      inputElem.addEventListener('focus', _ => {
        setColabEditing(true);
      });
      inputElem.addEventListener('blur', _ => {
        setColabEditing(false);
      });
      let updateTriggeredBySelect = false;
      inputElem.addEventListener('input', e => {
        if (!updateTriggeredBySelect) {
          update((e.target as HTMLInputElement).value);
          selectElem.value = (e.target as HTMLInputElement).value;
        }
        updateTriggeredBySelect = false;
      });
      // listen to the value change triggered by the dropdown menu
      pubsub.subscribe(
        `select-value${(key ?? '').toString()}`,
        (value: string) => {
          updateTriggeredBySelect = true;
          inputElem.value = value;
          inputElem.dispatchEvent(new Event('input'));
        }
      );

      selectElemWrapper.appendChild(inputElem);
    }

    return selectElemWrapper;
  } else if (
    targetType === BuildTargetType.InputStringOrRaw ||
    targetType === BuildTargetType.InputNumberOrIntegerOrDate
  ) {
    const inputElem = document.createElement('input');
    inputElem.classList.add(PARAM_EXTEND_ELEMENT_CLASS_NAME);
    inputElem.value =
      paramObj.config.type ===
      (targetType === BuildTargetType.InputStringOrRaw
        ? ParamType.String
        : ParamType.Date)
        ? paramObj.value ?? ''
        : paramObj.rawValue ?? '';
    paramObj.config.type === ParamType.Date && (inputElem.type = 'date');

    inputElem.addEventListener('focus', _ => {
      setColabEditing(true);
    });
    inputElem.addEventListener('blur', _ => {
      setColabEditing(false);
    });
    inputElem.addEventListener('input', e => {
      update((e.target as HTMLInputElement).value);
    });

    return inputElem;
  } else if (targetType === BuildTargetType.Checkbox) {
    const checkboxElem = document.createElement('input');
    checkboxElem.type = 'checkbox';
    checkboxElem.checked = paramObj.rawValue === 'True';
    checkboxElem.addEventListener('change', e => {
      setColabEditing(true);
      update((e.target as HTMLInputElement).checked ? 'True' : 'False');
      setColabEditing(false);
    });

    return checkboxElem;
  } else if (targetType === BuildTargetType.Slider) {
    const inputElem = document.createElement('input');
    inputElem.classList.add(PARAM_EXTEND_ELEMENT_CLASS_NAME);
    inputElem.value =
      paramObj.rawValue ||
      (paramObj.config.min && paramObj.config.min.toString()) ||
      (paramObj.config.max && paramObj.config.max.toString()) ||
      '0';
    inputElem.type = 'range';
    paramObj.config.min && (inputElem.min = paramObj.config.min.toString());
    paramObj.config.max && (inputElem.max = paramObj.config.max.toString());
    paramObj.config.step && (inputElem.step = paramObj.config.step.toString());

    inputElem.addEventListener('focus', _ => {
      setColabEditing(true);
    });
    inputElem.addEventListener('blur', _ => {
      setColabEditing(false);
    });
    inputElem.addEventListener('input', e => {
      update((e.target as HTMLInputElement).value);
    });

    return inputElem;
  }
};
