import * as colabCommand from './index';
import {
  CommandType,
  ERROR_MESSAGE_CLASS_NAME,
  ERROR_CLASS_NAME,
  DisplayMode,
  MARKDOWN_CLASS_NAME,
  QuoteType
} from '../constants';

const updateCell = jest.fn((value: string) => value);
const setColabEditing = jest.fn((isEditing: boolean) => isEditing);
const onClick = jest.fn((e: Event) => {
  e;
});

describe('test colabCommand', () => {
  beforeEach(() => {
    jest.resetAllMocks();
  });

  test('match', () => {
    expect(colabCommand.colabCommand.match('#@param')).toBe(CommandType.Param);
  });

  test('not match', () => {
    expect(colabCommand.colabCommand.match('#@pram')).toBeFalsy();
  });

  test('trim', () => {
    expect(
      colabCommand.colabCommand.trim(
        CommandType.Param,
        'aa = "123"#@param ["1", "123"]'
      )
    ).toEqual({
      name: 'aa',
      value: '123',
      rawValue: '"123"',
      valueList: ['1', '123'],
      config: undefined,
      quoteType: QuoteType.Double
    });
  });

  test('updateLine', () => {
    expect(
      colabCommand.colabCommand.updateLine(
        CommandType.Param,
        'aa = "123"#@param ["1", "123"]',
        '123',
        '1',
        QuoteType.Double
      )
    ).toEqual('aa = "1"#@param ["1", "123"]');
  });

  test('build with assert error', () => {
    const colabElem = colabCommand.colabCommand.build(
      CommandType.Param,
      'aa#@param',
      1,
      updateCell,
      setColabEditing
    );
    expect(
      colabElem?.classList.contains(ERROR_MESSAGE_CLASS_NAME)
    ).toBeTruthy();
  });

  test('build with param and no config', () => {
    const colabElem = colabCommand.colabCommand.build(
      CommandType.Param,
      'aa = "3"#@param',
      1,
      updateCell,
      setColabEditing
    );
    expect(colabElem?.querySelector('input')).toBeTruthy();
  });

  test('build with param and no config and cache', () => {
    let colabElem = colabCommand.colabCommand.build(
      CommandType.Param,
      'aa = "3"#@param',
      1,
      updateCell,
      setColabEditing
    );
    colabElem = colabCommand.colabCommand.build(
      CommandType.Param,
      'aa = "3"#@param',
      1,
      updateCell,
      setColabEditing
    );
    expect(colabElem?.querySelector('input')).toBeTruthy();
  });

  test('build with param and no type', () => {
    const colabElem = colabCommand.colabCommand.build(
      CommandType.Param,
      'aa = "3"#@param{}',
      1,
      updateCell,
      setColabEditing
    );
    const inputElem = colabElem?.querySelector('input');
    inputElem && (inputElem.value = '5');
    inputElem && inputElem.dispatchEvent(new Event('input'));
    expect(colabElem?.querySelector('input')).toBeTruthy();
  });

  test('build with param type = string and valueList', () => {
    const colabElem = colabCommand.colabCommand.build(
      CommandType.Param,
      'aa = "3"#@param["1", "2", "3"]',
      1,
      updateCell,
      setColabEditing
    );
    colabElem
      ?.querySelectorAll('option')[1]
      .setAttribute('selected', 'selected');
    colabElem?.querySelector('select')?.dispatchEvent(new Event('change'));
    expect(colabElem?.querySelector('select')).toBeTruthy();
    expect(colabElem?.querySelector('select')).toBeTruthy();
  });

  test('build with param type = string and valueList and single quotes', () => {
    const colabElem = colabCommand.colabCommand.build(
      CommandType.Param,
      'aa = \'3\'#@param["1", "2", "3"]',
      1,
      updateCell,
      setColabEditing
    );
    colabElem
      ?.querySelectorAll('option')[1]
      .setAttribute('selected', 'selected');
    colabElem?.querySelector('select')?.dispatchEvent(new Event('change'));
    expect(colabElem?.querySelector('select')).toBeTruthy();
    expect(colabElem?.querySelector('select')).toBeTruthy();
  });

  test('build with param type = string and valueList and no rawValue', () => {
    const colabElem = colabCommand.colabCommand.build(
      CommandType.Param,
      'aa =#@param["1", "2", "3"]',
      1,
      updateCell,
      setColabEditing
    );
    colabElem
      ?.querySelectorAll('option')[1]
      .setAttribute('selected', 'selected');
    colabElem?.querySelector('select')?.dispatchEvent(new Event('change'));
    expect(colabElem?.querySelector('select')).toBeTruthy();
    expect(colabElem?.querySelector('select')).toBeTruthy();
  });

  test('build with param type = raw', () => {
    const colabElem = colabCommand.colabCommand.build(
      CommandType.Param,
      'aa = "3"#@param {type: "raw"}',
      1,
      updateCell,
      setColabEditing
    );
    const inputElem = colabElem?.querySelector('input');
    inputElem && (inputElem.value = '4');
    inputElem && inputElem.dispatchEvent(new Event('input'));
    expect(colabElem?.querySelector('input')).toBeTruthy();
  });

  test('build with param type = raw and valueList', () => {
    const colabElem = colabCommand.colabCommand.build(
      CommandType.Param,
      'aa = "3"#@param {type: "raw"}["1", "2", "3"]',
      1,
      updateCell,
      setColabEditing
    );
    colabElem
      ?.querySelectorAll('option')[1]
      .setAttribute('selected', 'selected');
    colabElem?.querySelector('select')?.dispatchEvent(new Event('change'));
    expect(colabElem?.querySelector('select')).toBeTruthy();
  });

  test('build with param type = number', () => {
    const colabElem = colabCommand.colabCommand.build(
      CommandType.Param,
      'aa = 3#@param {type: "number"}',
      1,
      updateCell,
      setColabEditing
    );
    const inputElem = colabElem?.querySelector('input');
    inputElem && (inputElem.value = '4');
    inputElem && inputElem.dispatchEvent(new Event('input'));
    expect(colabElem?.querySelector('input')).toBeTruthy();
  });

  test('build with param type = integer', () => {
    const colabElem = colabCommand.colabCommand.build(
      CommandType.Param,
      'aa = 3#@param {type: "integer"}',
      1,
      updateCell,
      setColabEditing
    );
    expect(colabElem?.querySelector('input')).toBeTruthy();
  });

  test('build with param type = date', () => {
    const colabElem = colabCommand.colabCommand.build(
      CommandType.Param,
      'aa = "2003-10-03"#@param {type: "date"}',
      1,
      updateCell,
      setColabEditing
    );
    const inputElem = colabElem?.querySelector('input');
    inputElem && (inputElem.value = '2003-10-05');
    inputElem && inputElem.dispatchEvent(new Event('input'));
    expect(colabElem?.querySelector('input')).toBeTruthy();
  });

  test('build with param type = boolean', () => {
    const colabElem = colabCommand.colabCommand.build(
      CommandType.Param,
      'aa = True#@param {type: "boolean"}',
      1,
      updateCell,
      setColabEditing
    );
    const inputElem = colabElem?.querySelector('input');
    inputElem && (inputElem.checked = false);
    inputElem && inputElem.dispatchEvent(new Event('change'));
    expect(colabElem?.querySelector('input')).toBeTruthy();
  });

  test('build with param type = slider and too large value', () => {
    const colabElem = colabCommand.colabCommand.build(
      CommandType.Param,
      'aa = 10#@param {type: "slider", min: 1, max: 5}',
      1,
      updateCell,
      setColabEditing
    );
    const sliderElem = colabElem?.querySelector('input');
    sliderElem && (sliderElem.value = '10');
    sliderElem && sliderElem.dispatchEvent(new Event('input'));
    expect(colabElem?.querySelector('input')).toBeTruthy();
  });

  test('build with param type = slider and too small value', () => {
    const colabElem = colabCommand.colabCommand.build(
      CommandType.Param,
      'aa = -5#@param {type: "slider", min: 1, max: 5}',
      1,
      updateCell,
      setColabEditing
    );
    const sliderElem = colabElem?.querySelector('input');
    sliderElem && (sliderElem.value = '10');
    sliderElem && sliderElem.dispatchEvent(new Event('input'));
    expect(colabElem?.querySelector('input')).toBeTruthy();
  });

  test('build with param type = slider', () => {
    const colabElem = colabCommand.colabCommand.build(
      CommandType.Param,
      'aa = 10#@param {type: "slider"}',
      1,
      updateCell,
      setColabEditing
    );
    const sliderElem = colabElem?.querySelector('input');
    sliderElem && (sliderElem.value = '10');
    sliderElem && sliderElem.dispatchEvent(new Event('input'));
    expect(colabElem?.querySelector('input')).toBeTruthy();
  });

  test('build with param type = slider and wrong value', () => {
    const colabElem = colabCommand.colabCommand.build(
      CommandType.Param,
      'aa = 10#@param {type: "slider", min: 20}',
      1,
      updateCell,
      setColabEditing
    );
    const sliderElem = colabElem?.querySelector('input');
    sliderElem && (sliderElem.value = '10');
    sliderElem && sliderElem.dispatchEvent(new Event('input'));
    expect(colabElem?.querySelector('input')).toBeTruthy();
  });

  test('build with param wrong type', () => {
    const colabElem = colabCommand.colabCommand.build(
      CommandType.Param,
      'aa = 10#@param {type: "slide"}',
      1,
      updateCell,
      setColabEditing
    );
    expect(colabElem?.classList.contains(ERROR_CLASS_NAME)).toBeTruthy();
  });

  test('build with title', () => {
    const colabElem = colabCommand.colabCommand.build(
      CommandType.Title,
      'aa = 10#@title123',
      0,
      updateCell,
      setColabEditing
    );
    expect(colabElem).toBeTruthy();
  });

  test('build with error', () => {
    const colabElem = colabCommand.colabCommand.build(
      CommandType.Param,
      '',
      1,
      updateCell,
      setColabEditing
    );
    expect(
      colabElem?.classList.contains(ERROR_MESSAGE_CLASS_NAME)
    ).toBeTruthy();
  });
});

describe('other methods', () => {
  it('should parse display mode', () => {
    expect(
      colabCommand.colabCommand.parseDisplayMode("asd{display-mode: 'code'}")
    ).toEqual({
      displayMode: DisplayMode.Code,
      newLine: 'asd'
    });
  });

  it('should work without config', () => {
    expect(colabCommand.colabCommand.parseDisplayMode('asd')).toBeFalsy();
  });

  it('should create the show code btn', () => {
    expect(
      colabCommand.colabCommand
        .showCodeBtn('abc', onClick)
        .classList.contains('abc')
    ).toBeTruthy();
  });

  it('should create the toolbar code btns', () => {
    expect(
      colabCommand.colabCommand
        .toolbarBtns('abc', onClick, onClick)
        .classList.contains('abc')
    ).toBeTruthy();
  });
});

describe('markdown builder', () => {
  it('should work', () => {
    colabCommand.markdownBuilder.add('123');
    expect(colabCommand.markdownBuilder.get()).toBe('123');
    colabCommand.markdownBuilder.add('123');
    expect(colabCommand.markdownBuilder.get()).toBe('123\n123');
    colabCommand.markdownBuilder.clear();
    expect(colabCommand.markdownBuilder.get()).toBe('');
    colabCommand.markdownBuilder.add('> 123');
    colabCommand.markdownBuilder.add('> 123');
    expect(
      colabCommand.markdownBuilder
        .build()
        .classList.contains(MARKDOWN_CLASS_NAME)
    ).toBeTruthy();
  });
});
