import { marked } from 'marked';
import { localeText } from '../locale';
import {
  ERROR_MESSAGE_CLASS_NAME,
  SLIDER_VALUE_CLASS_NAME,
  PARAM_QUOTATION_CLASS_NAME,
  PARAM_CLASS_NAME,
  BuildTargetType,
  PARAM_NAME_CLASS_NAME,
  BREAK_LINE_ELEMENT,
  TITLE_CLASS_NAME,
  MARKDOWN_CLASS_NAME,
  JUPYTER_LAB_MARKDOWN_CLASS_NAME,
  CommandType,
  ParamType,
  LocaleTextName,
  ParamObj,
  DisplayMode,
  ERROR_CLASS_NAME,
  QuoteType,
  TOOLBAR_CODE_CLASS_NAME,
  TOOLBAR_FORM_CLASS_NAME
} from '../constants';
import { LRUCache } from '../utils/cache';
import { strToJSON } from '../utils/utils';
import { colabCommandTrimmer } from './trimmer';
import { colabCommandMatcher } from './matcher';
import { colabCommandUpdater } from './updater';
import { colabCommandBuilder } from './builder';

// wait until the jupyterlab sets the language code
setTimeout(() => localeText.setLang(document.documentElement.lang), 0);

// initialize the element cache with maxSize
const elemCache = new LRUCache(100);

const makeLegalString = (rawValue: string, value: string): string => {
  if (!(/'.*'/.exec(rawValue) || /".*"/.exec(rawValue))) {
    value = `"${value.replaceAll('"', '\\"')}"`;
  }
  return value;
};

// verify the number values
const numberValueAssert = {
  [ParamType.Number]: ({ value }: { value: string }) =>
    !isNaN(Number(value)) && !isNaN(parseFloat(value)),
  [ParamType.Slider]: ({
    value,
    min,
    max
  }: {
    value: string;
    min?: number;
    max?: number;
  }) => {
    if (
      numberValueAssert.number({ value }) &&
      (min !== undefined ? parseFloat(value) >= min : true) &&
      (max !== undefined ? parseFloat(value) <= max : true)
    ) {
      return true;
    }
    return false;
  },
  [ParamType.Integer]: ({ value }: { value: string }) =>
    Number.isInteger(Number(value))
};

// error messages for corresponding number type
const numberValueErrorMsg = {
  [ParamType.Number]: localeText.getText(LocaleTextName.NUMBER_ERROR_MESSAGE),
  [ParamType.Slider]: localeText.getText(LocaleTextName.SLIDER_ERROR_MESSAGE),
  [ParamType.Integer]: localeText.getText(LocaleTextName.INTEGER_ERROR_MESSAGE)
};

// build the error message DOM element with the error message
const createErrorMsgElem = (msg: string) => {
  const errorMsgElem = document.createElement('div');
  errorMsgElem.classList.add(ERROR_MESSAGE_CLASS_NAME);
  errorMsgElem.innerHTML = msg;
  return errorMsgElem;
};

// build the error message DOM element with the error message
const createBreakLineElem = () => {
  const brElem = document.createElement('div');
  brElem.classList.add(BREAK_LINE_ELEMENT);
  return brElem;
};

// generic assert function to make sure the required fields are populated
const assert = (obj: Record<string, any>, ...fields: (keyof ParamObj)[]) => {
  for (const field of fields) {
    if (obj[field] === null || obj[field] === undefined) {
      return localeText.getTextFn(LocaleTextName.ASSERT_ERROR_MESSAGE_FN)(
        field
      );
    }
  }
};

export const colabCommand = {
  match(line: string): CommandType | undefined {
    for (const commandType of Object.values(CommandType)) {
      if (colabCommandMatcher(commandType, line)) {
        return commandType;
      }
    }
  },
  trim(type: CommandType, line: string) {
    return colabCommandTrimmer[type](line);
  },
  updateLine(
    type: CommandType.Param,
    line: string,
    replaceStr: string | undefined,
    withStr: string,
    quoteType: QuoteType | undefined,
    needQuotation = false
  ): string {
    return colabCommandUpdater[type](
      line,
      replaceStr,
      withStr,
      quoteType,
      needQuotation
    );
  },
  build(
    type: CommandType,
    line: string,
    lineNumber: number,
    updateCell: (line: string) => void,
    setColabEditing: (isEditing: boolean) => void
  ): HTMLElement | undefined {
    try {
      const cachedElem = elemCache.get(
        line + lineNumber.toString()
      ) as HTMLElement;
      if (cachedElem) {
        return cachedElem;
      }

      const colabElem = document.createElement('div');

      const validateValue = (
        paramType: ParamType.Number | ParamType.Integer | ParamType.Slider,
        value:
          | { value: string }
          | { value: string; max?: number; min?: number },
        clearPreviousError?: boolean
      ) => {
        if (clearPreviousError) {
          colabElem.querySelector('.' + ERROR_MESSAGE_CLASS_NAME)?.remove();
          colabElem.querySelector('.' + BREAK_LINE_ELEMENT)?.remove();
          colabElem.classList.remove(ERROR_CLASS_NAME);
        }
        if (
          numberValueAssert[paramType] &&
          !numberValueAssert[paramType](value)
        ) {
          const errorMsgElem = createErrorMsgElem(
            numberValueErrorMsg[paramType]
          );
          colabElem.appendChild(createBreakLineElem());
          colabElem.appendChild(errorMsgElem);
          colabElem.classList.add(ERROR_CLASS_NAME);
        }
      };

      const trimmedObj = this.trim(type, line);

      // assertions
      if (type === CommandType.Param) {
        const paramObj = trimmedObj as ParamObj;
        if (assert(paramObj, 'name')) {
          const errorMsg = document.createElement('div');
          errorMsg.classList.add(ERROR_MESSAGE_CLASS_NAME);
          errorMsg.innerHTML = assert(paramObj, 'name') as string;
          return errorMsg;
        }
      }

      // build the form
      if (type === CommandType.Param) {
        const paramObj = trimmedObj as ParamObj;
        // variable name element
        const varNameElem = document.createElement('span');
        varNameElem.innerHTML = paramObj.name + ':';
        varNameElem.classList.add(PARAM_NAME_CLASS_NAME);

        // if no type is provided, set it to string by default
        if (!paramObj.config) {
          paramObj.config = { type: ParamType.String };
        }
        if (!paramObj.config.type) {
          paramObj.config.type = ParamType.String;
        }

        // string and raw share most code, except when you need to use the value
        if (
          paramObj.config.type === ParamType.String ||
          paramObj.config.type === ParamType.Raw
        ) {
          // build the dropdown element if valueList is provided
          if (paramObj.valueList) {
            const selectElemWrapper = colabCommandBuilder(
              BuildTargetType.ValueList,
              paramObj,
              (value: string) => {
                updateCacheAndCell(
                  this.updateLine(
                    type,
                    line,
                    paramObj.config.type === ParamType.String
                      ? paramObj.value
                      : paramObj.rawValue,
                    paramObj.config.type === ParamType.String
                      ? makeLegalString(paramObj.rawValue!, value)
                      : value,
                    paramObj.quoteType,
                    paramObj.config.type === ParamType.String
                  )
                );
              },
              setColabEditing,
              lineNumber
            );

            colabElem.appendChild(varNameElem);
            selectElemWrapper && colabElem.appendChild(selectElemWrapper);
          } else {
            const inputElem = colabCommandBuilder(
              BuildTargetType.InputStringOrRaw,
              paramObj,
              (value: string) => {
                updateCacheAndCell(
                  this.updateLine(
                    type,
                    line,
                    paramObj.config.type === ParamType.String
                      ? paramObj.value
                      : paramObj.rawValue,
                    paramObj.config.type === ParamType.String
                      ? makeLegalString(paramObj.rawValue!, value)
                      : value,
                    paramObj.quoteType,
                    paramObj.config.type === ParamType.String
                  )
                );
              },
              setColabEditing
            );

            const quotationElem = document.createElement('span');
            quotationElem.innerHTML = '"';
            quotationElem.classList.add(PARAM_QUOTATION_CLASS_NAME);

            colabElem.appendChild(varNameElem);
            paramObj.config.type === ParamType.String &&
              colabElem.appendChild(quotationElem);
            inputElem && colabElem.appendChild(inputElem);
            paramObj.config.type === ParamType.String &&
              colabElem.appendChild(quotationElem.cloneNode(true));
          }
        } else if (
          paramObj.config.type === ParamType.Number ||
          paramObj.config.type === ParamType.Integer ||
          paramObj.config.type === ParamType.Date
        ) {
          const paramType = paramObj.config.type;
          const inputElem = colabCommandBuilder(
            BuildTargetType.InputNumberOrIntegerOrDate,
            paramObj,
            (value: string) => {
              paramType !== ParamType.Date &&
                validateValue(paramType, { value }, true);
              updateCacheAndCell(
                this.updateLine(
                  type,
                  line,
                  paramObj.config.type === ParamType.Date
                    ? paramObj.value
                    : paramObj.rawValue,
                  value,
                  paramObj.quoteType,
                  true
                )
              );
            },
            setColabEditing
          );

          colabElem.appendChild(varNameElem);
          inputElem && colabElem.appendChild(inputElem);

          // verify the number format after the init process
          paramType !== ParamType.Date &&
            validateValue(paramType, {
              value: (inputElem as HTMLInputElement).value
            });
        } else if (paramObj.config.type === ParamType.Boolean) {
          const checkboxElem = colabCommandBuilder(
            BuildTargetType.Checkbox,
            paramObj,
            (value: string) => {
              updateCacheAndCell(
                this.updateLine(
                  type,
                  line,
                  paramObj.rawValue,
                  value,
                  paramObj.quoteType
                )
              );
            },
            setColabEditing
          );

          colabElem.appendChild(varNameElem);
          checkboxElem && colabElem.appendChild(checkboxElem);
        } else if (paramObj.config.type === ParamType.Slider) {
          const paramType = paramObj.config.type as ParamType.Slider;
          const createSliderValueElem = (value: string) => {
            const valueElem = document.createElement('span');
            valueElem.classList.add(SLIDER_VALUE_CLASS_NAME);
            valueElem.innerHTML = value;
            return valueElem;
          };

          const valueElem = createSliderValueElem(paramObj.rawValue!);

          const inputElem = colabCommandBuilder(
            BuildTargetType.Slider,
            paramObj,
            (value: string) => {
              validateValue(
                paramType,
                {
                  value: value,
                  min: paramObj.config.min,
                  max: paramObj.config.max
                },
                true
              );

              valueElem.innerHTML = value;
              const oldValueElem = colabElem.querySelector(
                '.' + SLIDER_VALUE_CLASS_NAME
              );
              const newValueElem = createSliderValueElem(value);
              oldValueElem &&
                colabElem.replaceChild(newValueElem, oldValueElem);

              updateCacheAndCell(
                this.updateLine(
                  type,
                  line,
                  paramObj.rawValue,
                  value,
                  paramObj.quoteType
                )
              );
            },
            setColabEditing
          );

          colabElem.appendChild(varNameElem);
          inputElem && colabElem.appendChild(inputElem);
          colabElem.appendChild(valueElem);

          // verify the number format after the init process
          validateValue(paramType, {
            value: paramObj.rawValue!,
            min: Number(paramObj.config.min),
            max: Number(paramObj.config.max)
          });
        } else {
          const errorMsgElem = createErrorMsgElem(
            localeText.getTextFn(LocaleTextName.CELL_BUILD_ERROR_MESSAGE_FN)(
              line
            )
          );
          colabElem.appendChild(errorMsgElem);
          colabElem.classList.add(ERROR_CLASS_NAME);
        }

        colabElem.classList.add(PARAM_CLASS_NAME);
      } else if (type === CommandType.Title) {
        const titleElem = document.createElement('div');
        titleElem.classList.add(JUPYTER_LAB_MARKDOWN_CLASS_NAME);
        titleElem.innerHTML = marked.parse(
          (trimmedObj as { value: string }).value
        );

        colabElem.classList.add(TITLE_CLASS_NAME);
        colabElem.appendChild(titleElem);
      }

      const updateCacheAndCell = (newLine: string) => {
        elemCache.remove(line + lineNumber.toString());
        elemCache.set(newLine + lineNumber.toString(), colabElem);
        updateCell(newLine);
      };
      elemCache.set(line + lineNumber.toString(), colabElem);
      return colabElem;
    } catch (e) {
      return createErrorMsgElem(
        localeText.getTextFn(LocaleTextName.CELL_BUILD_ERROR_MESSAGE_FN)(line)
      );
    }
  },
  // displayMode is a special case because it's like an additional command
  // so we handle it outside of the normal build process
  parseDisplayMode(
    line: string
  ): { displayMode: DisplayMode; newLine: string } | undefined {
    const displayModeStr = [...line.matchAll(/{.*?}/g)][0]?.pop();
    const newLine = line.slice(0, line.indexOf(displayModeStr ?? ''));
    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
    const displayModeConfig: Record<string, string> | undefined =
      displayModeStr &&
      JSON.parse(
        strToJSON(
          // wrap property names with double quotations
          displayModeStr
            .replaceAll("'", '"') // replace existing single quotations with double quotations
            .replace('display-mode', 'displayMode') // strToJSON couldn't handle the dash, remove it first
        )
      );
    if (displayModeConfig) {
      return {
        displayMode: displayModeConfig.displayMode as DisplayMode,
        newLine
      };
    }
  },
  // build toolBar buttons
  toolbarBtns(
    containerClassName: string,
    codeBtnOnClick: EventListener,
    formBtnOnClick: EventListener
  ): HTMLElement {
    const container = document.createElement('div');
    container.classList.add(containerClassName);

    const codeBtnElem = document.createElement('a');
    codeBtnElem.classList.add(TOOLBAR_CODE_CLASS_NAME);
    codeBtnElem.addEventListener('click', codeBtnOnClick);
    codeBtnElem.innerHTML =
      '<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 640 512"><!--! Font Awesome Pro 6.2.0 by @fontawesome - https://fontawesome.com License - https://fontawesome.com/license (Commercial License) Copyright 2022 Fonticons, Inc. --><path d="M392.8 1.2c-17-4.9-34.7 5-39.6 22l-128 448c-4.9 17 5 34.7 22 39.6s34.7-5 39.6-22l128-448c4.9-17-5-34.7-22-39.6zm80.6 120.1c-12.5 12.5-12.5 32.8 0 45.3L562.7 256l-89.4 89.4c-12.5 12.5-12.5 32.8 0 45.3s32.8 12.5 45.3 0l112-112c12.5-12.5 12.5-32.8 0-45.3l-112-112c-12.5-12.5-32.8-12.5-45.3 0zm-306.7 0c-12.5-12.5-32.8-12.5-45.3 0l-112 112c-12.5 12.5-12.5 32.8 0 45.3l112 112c12.5 12.5 32.8 12.5 45.3 0s12.5-32.8 0-45.3L77.3 256l89.4-89.4c12.5-12.5 12.5-32.8 0-45.3z"/></svg>';

    const divider = document.createElement('div');
    divider.innerHTML = '|';

    const formBtnElem = document.createElement('a');
    formBtnElem.classList.add(TOOLBAR_FORM_CLASS_NAME);
    formBtnElem.addEventListener('click', formBtnOnClick);
    formBtnElem.innerHTML =
      '<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 640 512"><!--! Font Awesome Pro 6.2.0 by @fontawesome - https://fontawesome.com License - https://fontawesome.com/license (Commercial License) Copyright 2022 Fonticons, Inc. --><path d="M601.9 16c-18.8-18.8-49.2-18.7-67.9 0L494.4 55.7l89.3 89.3 39.8-38.9c19.1-18.7 19.2-49.4 .4-68.2L601.9 16zM328.5 221.8c-7 7-12.3 15.6-15.4 25l-23.8 71.3c-2.9 8.6-.6 18.1 5.8 24.6s15.9 8.7 24.6 5.8l71.5-23.9c9.2-3.1 17.6-8.2 24.5-15L560.8 167.5 471.7 78.4 328.5 221.8zM80 128c-44.2 0-80 35.8-80 80V432c0 44.2 35.8 80 80 80H496c44.2 0 80-35.8 80-80V304c0-17.7-14.3-32-32-32s-32 14.3-32 32V432c0 8.8-7.2 16-16 16H80c-8.8 0-16-7.2-16-16V208c0-8.8 7.2-16 16-16H272c17.7 0 32-14.3 32-32s-14.3-32-32-32H80zm80 192c0-17.7-14.3-32-32-32s-32 14.3-32 32s14.3 32 32 32s32-14.3 32-32zm64 32c17.7 0 32-14.3 32-32s-14.3-32-32-32s-32 14.3-32 32s14.3 32 32 32z"/></svg>';

    container.appendChild(codeBtnElem);
    container.appendChild(divider);
    container.appendChild(formBtnElem);

    return container;
  },
  showCodeBtn(className: string, onClick: EventListener): HTMLElement {
    const btnElem = document.createElement('a');
    btnElem.classList.add(className);
    btnElem.innerHTML = localeText.getText(LocaleTextName.SHOW_CODE_BTN_TEXT);
    btnElem.addEventListener('click', onClick);

    return btnElem;
  }
};

// handle markdown separately because we need to support multiple lines parsing
let markdownRaw = '';

export const markdownBuilder = {
  add(content: string): void {
    if (markdownRaw) {
      markdownRaw += '\n' + content;
    } else {
      markdownRaw = content;
    }
  },
  clear(): void {
    markdownRaw = '';
  },
  get(): string {
    return markdownRaw;
  },
  build(): HTMLElement {
    const markdownElem = document.createElement('div');
    markdownElem.classList.add(
      MARKDOWN_CLASS_NAME,
      JUPYTER_LAB_MARKDOWN_CLASS_NAME
    );
    markdownElem.innerHTML = marked.parse(markdownRaw);
    return markdownElem;
  }
};
