import { colabCommandTrimmer } from './trimmer';
import { CommandType, QuoteType } from '../constants';

describe('test title trimmer', () => {
  test('should trim the title', () => {
    expect(colabCommandTrimmer[CommandType.Title]('#@title123')).toEqual({
      value: '123'
    });
    expect(colabCommandTrimmer[CommandType.Title]('#@title 123')).toEqual({
      value: '123'
    });
    expect(colabCommandTrimmer[CommandType.Title]('#@title 123 ')).toEqual({
      value: '123'
    });
  });
});

describe('test markdown trimmer', () => {
  test('should trim the markdown', () => {
    expect(colabCommandTrimmer[CommandType.Markdown]('#@markdown123')).toEqual(
      '123'
    );
    expect(colabCommandTrimmer[CommandType.Markdown]('#@markdown 123')).toEqual(
      '123'
    );
    expect(
      colabCommandTrimmer[CommandType.Markdown]('#@markdown 123 ')
    ).toEqual('123');
  });
});

describe('test param trimmer code part', () => {
  test('should trim the param without code', () => {
    expect(colabCommandTrimmer[CommandType.Param]('#@param')).toEqual({
      name: undefined,
      value: undefined,
      rawValue: undefined,
      config: undefined,
      valueList: undefined
    });

    expect(colabCommandTrimmer[CommandType.Param](' #@param')).toEqual({
      name: undefined,
      value: undefined,
      rawValue: undefined,
      config: undefined,
      valueList: undefined
    });

    expect(colabCommandTrimmer[CommandType.Param](' #@param ')).toEqual({
      name: undefined,
      value: undefined,
      rawValue: undefined,
      config: undefined,
      valueList: undefined
    });
  });

  test('should trim the param without =', () => {
    expect(colabCommandTrimmer[CommandType.Param]('aa#@param')).toEqual({
      name: undefined,
      value: undefined,
      rawValue: undefined,
      config: undefined,
      valueList: undefined
    });

    expect(colabCommandTrimmer[CommandType.Param]('aa #@param')).toEqual({
      name: undefined,
      value: undefined,
      rawValue: undefined,
      config: undefined,
      valueList: undefined
    });

    expect(colabCommandTrimmer[CommandType.Param]('aa #@param ')).toEqual({
      name: undefined,
      value: undefined,
      rawValue: undefined,
      config: undefined,
      valueList: undefined
    });
  });

  test('should trim the param with = and without value', () => {
    expect(colabCommandTrimmer[CommandType.Param]('aa=#@param')).toEqual({
      name: 'aa',
      value: '',
      rawValue: '',
      config: undefined,
      valueList: undefined
    });

    expect(colabCommandTrimmer[CommandType.Param]('aa= #@param')).toEqual({
      name: 'aa',
      value: '',
      rawValue: '',
      config: undefined,
      valueList: undefined
    });

    expect(colabCommandTrimmer[CommandType.Param]('aa= #@param ')).toEqual({
      name: 'aa',
      value: '',
      rawValue: '',
      config: undefined,
      valueList: undefined
    });

    expect(colabCommandTrimmer[CommandType.Param]('aa = #@param ')).toEqual({
      name: 'aa',
      value: '',
      rawValue: '',
      config: undefined,
      valueList: undefined
    });

    expect(colabCommandTrimmer[CommandType.Param]('aa =#@param ')).toEqual({
      name: 'aa',
      value: '',
      rawValue: '',
      config: undefined,
      valueList: undefined
    });
  });

  test('should trim the param with = and empty value', () => {
    expect(colabCommandTrimmer[CommandType.Param]("aa=''#@param")).toEqual({
      name: 'aa',
      value: '',
      rawValue: "''",
      config: undefined,
      valueList: undefined,
      quoteType: QuoteType.Single
    });

    expect(colabCommandTrimmer[CommandType.Param]("aa = ''#@param")).toEqual({
      name: 'aa',
      value: '',
      rawValue: "''",
      config: undefined,
      valueList: undefined,
      quoteType: QuoteType.Single
    });

    expect(colabCommandTrimmer[CommandType.Param]('aa=""#@param')).toEqual({
      name: 'aa',
      value: '',
      rawValue: '""',
      config: undefined,
      valueList: undefined,
      quoteType: QuoteType.Double
    });

    expect(colabCommandTrimmer[CommandType.Param]('aa = ""#@param')).toEqual({
      name: 'aa',
      value: '',
      rawValue: '""',
      config: undefined,
      valueList: undefined,
      quoteType: QuoteType.Double
    });
  });

  test('should trim the param with = and value', () => {
    expect(colabCommandTrimmer[CommandType.Param]('aa=123#@param')).toEqual({
      name: 'aa',
      value: '123',
      rawValue: '123',
      config: undefined,
      valueList: undefined
    });

    expect(colabCommandTrimmer[CommandType.Param]('aa=True#@param')).toEqual({
      name: 'aa',
      value: 'True',
      rawValue: 'True',
      config: undefined,
      valueList: undefined
    });

    expect(colabCommandTrimmer[CommandType.Param]("aa='True'#@param")).toEqual({
      name: 'aa',
      value: 'True',
      rawValue: "'True'",
      config: undefined,
      valueList: undefined,
      quoteType: QuoteType.Single
    });

    expect(colabCommandTrimmer[CommandType.Param]("aa='123'#@param")).toEqual({
      name: 'aa',
      value: '123',
      rawValue: "'123'",
      config: undefined,
      valueList: undefined,
      quoteType: QuoteType.Single
    });

    expect(colabCommandTrimmer[CommandType.Param]('aa="123"#@param')).toEqual({
      name: 'aa',
      value: '123',
      rawValue: '"123"',
      config: undefined,
      valueList: undefined,
      quoteType: QuoteType.Double
    });
  });
});

describe('test param trimmer colab part', () => {
  test('config', () => {
    expect(
      colabCommandTrimmer[CommandType.Param]('aa="123"#@param{type:"string"}')
    ).toEqual({
      name: 'aa',
      value: '123',
      rawValue: '"123"',
      config: {
        type: 'string'
      },
      valueList: undefined,
      quoteType: QuoteType.Double
    });

    expect(
      colabCommandTrimmer[CommandType.Param]('aa="123"#@param{type:"string"')
    ).toEqual({
      name: 'aa',
      value: '123',
      rawValue: '"123"',
      config: undefined,
      valueList: undefined,
      quoteType: QuoteType.Double
    });

    expect(
      colabCommandTrimmer[CommandType.Param](
        'aa="123"#@param {type: "string", allow-input: true}'
      )
    ).toEqual({
      name: 'aa',
      value: '123',
      rawValue: '"123"',
      config: {
        type: 'string',
        allowInput: true
      },
      valueList: undefined,
      quoteType: QuoteType.Double
    });

    expect(
      colabCommandTrimmer[CommandType.Param](
        'aa="123"#@param {type: "raw", allow-input: true}'
      )
    ).toEqual({
      name: 'aa',
      value: '123',
      rawValue: '"123"',
      config: {
        type: 'raw',
        allowInput: true
      },
      valueList: undefined,
      quoteType: QuoteType.Double
    });

    expect(
      colabCommandTrimmer[CommandType.Param]('aa="123"#@param {type: "date"}')
    ).toEqual({
      name: 'aa',
      value: '123',
      rawValue: '"123"',
      config: {
        type: 'date'
      },
      valueList: undefined,
      quoteType: QuoteType.Double
    });

    expect(
      colabCommandTrimmer[CommandType.Param]('aa="123"#@param {type: "number"}')
    ).toEqual({
      name: 'aa',
      value: '123',
      rawValue: '"123"',
      config: {
        type: 'number'
      },
      valueList: undefined,
      quoteType: QuoteType.Double
    });

    expect(
      colabCommandTrimmer[CommandType.Param](
        'aa="123"#@param {type: "slider", min: 0, max: 100, step: 1}'
      )
    ).toEqual({
      name: 'aa',
      value: '123',
      rawValue: '"123"',
      config: {
        type: 'slider',
        min: 0,
        max: 100,
        step: 1
      },
      valueList: undefined,
      quoteType: QuoteType.Double
    });
  });

  test('valueList', () => {
    expect(
      colabCommandTrimmer[CommandType.Param]('aa="123"#@param [1, 2, 3]')
    ).toEqual({
      name: 'aa',
      value: '123',
      rawValue: '"123"',
      config: undefined,
      valueList: [1, 2, 3],
      quoteType: QuoteType.Double
    });

    expect(
      colabCommandTrimmer[CommandType.Param]('aa="123"#@param ["1", "2", 3]')
    ).toEqual({
      name: 'aa',
      value: '123',
      rawValue: '"123"',
      config: undefined,
      quoteType: QuoteType.Double,
      valueList: ['1', '2', 3]
    });
  });
});
